import 'dart:io';
import 'dart:convert';
import 'package:get/get.dart';
import 'package:naiyouwl/app/clash/mode/config.dart';
import 'package:naiyouwl/app/clash/mode/clash_config.dart';
import 'package:naiyouwl/app/clash/service/clash_service.dart';
import 'package:naiyouwl/app/const/const.dart';
import 'package:naiyouwl/app/controller/controllers.dart';
import 'package:path/path.dart' as path;
import 'package:yaml/yaml.dart';
import 'package:yaml_edit/yaml_edit.dart';
import 'package:flutter_emoji/flutter_emoji.dart';

final Map<String, dynamic> _defaultConfig = {
  'selected': 'example.yaml',
  'updateInterval': 86400,
  'updateSubsAtStart': false,
  'setSystemProxy': false,
  'startAtLogin': false,
  'breakConnections': false,
  'language': 'zh_CN',
  'servicePort': 9899,
};

class ConfigController extends GetxController {
  var config = GuiConfig.fromJson(_defaultConfig).obs;

  var clashCoreApiAddress = '127.0.0.1:9799'.obs;
  var clashCoreApiSecret = ''.obs;
  var clashCoreDns = ''.obs;
  var clashCoreTunEnable = false.obs;
  var mixedPort = 9788.obs;
  var apiAddressPort = 9799.obs;

  Future<void> initConfig() async {
    // 创建用户配置目录
    if (!await Paths.config.exists()) await Paths.config.create(recursive: true);

    // 复制资源文件到配置目录
    await _copyAssetToConfig(Files.assetsCountryMmdb, Files.configCountryMmdb);
    await _copyAssetToConfig(Files.assetsGeoIP, Files.configGeoIP);
    await _copyAssetToConfig(Files.assetsGeosite, Files.configGeosite);
    if (Platform.isWindows) {
      await _copyAssetToConfig(Files.assetsWintun, Files.configWintun);
    }

    // 设置默认语言
    final locale = Get.deviceLocale!;
    _defaultConfig['language'] = '${locale.languageCode}_${locale.countryCode}';

    // 读取或创建配置文件
    if (await Files.configConfig.exists()) {
      final local = json.decode(await Files.configConfig.readAsString());
      config.value = GuiConfig.fromJson({..._defaultConfig, ...local});
    } else {
      config.value = GuiConfig.fromJson(_defaultConfig);
    }

    // 保存 GuiConfig 到 gui_config.json
    await saveGuiConfig();

    await makeInitConfig();
  }
  // 返回一个可修改的 GUI 配置对象
  GuiConfig getEditableGuiConfig() {
    return config.value;
  }

  // 更新 GUI 配置
  Future<void> updateGuiConfig(GuiConfig updatedConfig) async {
    config.value = updatedConfig;
    await saveGuiConfig();
  }

  // 示例使用方法：
  // var editableConfig = getEditableGuiConfig();
  // editableConfig.language = 'en_US';
  // editableConfig.startAtLogin = true;
  // await updateGuiConfig(editableConfig);



  Future<void> saveGuiConfig() async {
    await Files.configConfig.writeAsString(json.encode(config.value.toJson()));
  }

  Future<void> _copyAssetToConfig(File source, File destination) async {
    if (!await destination.exists()) {
      await source.copy(destination.path);
    }
  }

  Future<void> readClashCoreApi() async {
    final configStr = await File(path.join(Paths.config.path, config.value.selected)).readAsString();
    final yamlEditor = YamlEditor(configStr);
    final configMap = yamlEditor.parseAt([]) as YamlMap;

    clashCoreApiAddress.value = configMap['external-controller'] as String? ?? '127.0.0.1:9090';
    print("clash api address ${clashCoreApiAddress.value}");
    clashCoreApiSecret.value = (configMap['secret'] as String? ?? '');
    mixedPort.value = (configMap['mixed-port'] as int? ?? 9788);
    clashCoreTunEnable.value = (configMap['tun'] as YamlMap?)?['enable'] as bool == true;
    clashCoreDns.value = '';
    if ((configMap['dns'] as YamlMap?)?['enable'] as bool == true &&
        ((configMap['dns'] as YamlMap?)?['listen'] as String? ?? '').isNotEmpty) {
      final dns = ((configMap['dns'] as YamlMap)['listen'] as String? ?? '').split(':');
      if (dns.length == 2) {
        final ip = dns[0];
        final port = dns[1];
        if (port == '53') {
          clashCoreDns.value = ip == '0.0.0.0' ? '127.0.0.1' : ip;
        }
      }
    }
  }

  Future<void> makeInitConfig() async {
    var mode = Get.find<Controllers>().global.modesSelect.value;
    
    var clashConfig = ClashConfig(
      mixedPort: mixedPort.value,
      allowLan: true,
      bindAddress: '*',
      mode: mode,
      logLevel: 'debug',
      externalController: '127.0.0.1:${apiAddressPort.value}',
      unifiedDelay: false,
      geodataMode: true,
      tcpConcurrent: false,
      findProcessMode: 'strict',
      globalClientFingerprint: 'chrome',
      proxies: [],
      rules: [
        'GEOIP,CN,DIRECT',
        'MATCH,DIRECT'
      ], dns: null
    );

    final initConfigFile = File(path.join(Paths.config.path, 'init.yaml'));
    await initConfigFile.writeAsString(clashConfig.toYaml());
  }

  // ... 其他方法保持不变
}
